import re

# API related exceptions


class BadRequest(ValueError):
    pass


class PPAServerError(ValueError):
    pass


# Event (SMTP, syslog) related exceptions


class EventConfigurationError(Exception):
    pass


class EventNotConfigured(Exception):
    def __init__(self, event_type, _):
        super(EventNotConfigured, self).__init__(
            f"Cannot send {event_type.event_name} as {event_type.protocol} "
            "is either disabled or not configured in the PPA appliance."
        )


class EventTypeDisabled(Exception):
    def __init__(self, event_type, _):
        super(EventTypeDisabled, self).__init__(
            f"Cannot send {event_type.event_name} as {event_type.protocol} is disabled in the PPA appliance."
        )


class NoMailRecipients(Exception):
    def __init__(self, _, __):
        super(NoMailRecipients, self).__init__(
            "Cannot send email as no recipients were provided in the API call, "
            "and the PPA appliance SMTP configuration has no default recipient."
        )


class EmailEventError(Exception):
    def __init__(self, _, response_text):

        if "could not connect to server" in response_text:
            message = "Failed to send email as the connection to the SMTP server failed."
        elif "no address" in response_text:
            message = "Failed to send email as one or more of the recipient addresses is empty."
        elif "missing '@' or" in response_text:
            message = "Failed to send email as one or more of the recipient addresses is not in a valid format."
        elif "cannot unmarshal" in response_text:
            # Example error below:
            #
            #   json: cannot unmarshal number into Go struct field smtpMessage.html of type string
            match = re.search(r"struct field smtpMessage\.(\w+)\s", response_text)
            if match is not None:
                message = f"Failed to send email as the {match.group(1)} parameter is in the wrong format."
            else:
                message = f"Failed to send email as one of the parameters is in the wrong format: {response_text}."
        else:
            message = f"Failed to send email: {response_text}"
        super(EmailEventError, self).__init__(message)


# Appliance version exceptions


class RetiredFunction(Exception):
    pass


class PPAVersionError(Exception):
    pass


# Data exceptions


class ValueAlreadyExists(Exception):
    pass
