from typing import Callable

from ppa import ui, appliance, exceptions


# The min_ppa_version decorator in SDK doesn't display the error, this wrapped version does.
def min_ppa_version(minimum_version: str) -> Callable:
    def _wrapped_version_check(func):
        def _execute(*args, **kwargs):
            try:
                return appliance.min_ppa_version(minimum_version)(func)(*args, **kwargs)
            except exceptions.PPAVersionError as e:
                ui.output_error(str(e))
                raise e

        return _execute

    return _wrapped_version_check
